#! /usr/bin/ipstcl

######################################################################
# Copyright 2001 Nokia IP, Inc.  All rights reserved.
#
# YOU MAY ONLY USE THE MATERIALS FOUND IN THIS FILE PURSUANT TO THE
# TERMS OF THE EVALUATION LICENSE AGREEMENT FOUND IN "/LICENSE".  ALL
# OTHER USES ARE EXPRESSLY PROHIBITED.
######################################################################
#
# $Id: aaa_servers.tcl,v 1.7 2002/10/03 21:57:14 laura Exp $
#

# CGI Tcl script for configuring servers to be used by the
# Authentication, Authorization and Accounting (AAA) feature.

################################################################
# Common procedure definitions
################################################################

# luniq -- Given a list 'l', find any adjacent elements that are identical
# and reduce them to one.  So {a b b c d d d d c} becomes {a b c d c}.
proc luniq {l} {
    set r {} ; # building a list to return
    set llast {} ; # the last item seen
    set llast_any 0 ; # boolean: does $llast's value exist?

    foreach e $l {
	if {$llast_any && $llast == $e} {
	    # this is a duplicate
	    continue
	}
	set llast_any 1
	set llast $e
	lappend r $e
    }

    return $r
}

# arraymap_* -- Tcl's "array" functions stink in subtle ways.  It's
# really annoying that 'array get' uses pattern matching, so if I have
# a variable x and want to use it as a lookup key in 'array get a $x' I
# have to worry about special characters in $x.  To get around this issue
# and to generally make things more convenient I'm creating these
# arraymap_* functions.
#	arraymap_set <array> <key> <value>
#		Maps <key> to <value> in the named array; destroys any
#		previous value it had.
#	arraymap_listall <array>
#		Returns all entries in array with
#		name and value alternating.
#	arraymap_get <array> <name>
#		Returns the value that <name> is mapped to in <array>.

proc arraymap_set {ary key val} {
    global $ary
    array set $ary [list $key $val]
}

proc arraymap_listall {ary} {
    global $ary
    return [array get $ary]
}

proc arraymap_get {ary name} {
    global $ary

    # Make a pattern out of name.
    set mname $name
    regsub -all {[][*?\\]} $mname {\\\0} mname

    foreach {gotn gotv} [array get $ary "${mname}*"] {
	if {$gotn == $name} {
	    return $gotv
	}
    }
}

# We restrict 'secret' to not contain a backslash.  (The underlying
# system has no trouble with the backslash, but "libdb set -list" has
# some trouble.)  One other character is forbidden: newline.  But
# that is checked in the CDK, not in this script, because it's a
# problem that appears at a "lower level."
proc check_for_backslash {val} {
    global err_list
    global result

    if {[string first "\\" $val] != -1} {
	set ev "'Shared secret' may not contain a backslash in this version of"
	set ev "$ev Voyager."
	lappend err_list $ev
	set result 1
    }
}

################################################################
# End, common procedure definitions
################################################################

source check_cluster.tcl
ipso

# define subroutines
source aaa_subs.tcl
source validate.tcl
source show_result.tcl

set err_list [list]

set UP "aaa.tcl"
set TOP main

set is_cluster [check_cluster]
#cid will be -1 if is_cluster is false
set cid [get_cluster_id]


# When getting a "shared secret" we want to allow users to enter at least
# 16 chars.  This involves setting "-maxlen" to something more than the default
# of 14.  Also, some browsers only allow up to "width" characters in a field;
# for those browsers we enlarge the width from its default of 14.
set secret_maxlen 256
set secret_width 16

# Set title -- for now, just a dull but convenient default; may be set
# to something more specific, below.
if {$is_cluster} {
    set page_title "Cluster AAA Servers Configuration"
} else {
    set page_title "AAA Servers Configuration"
}
set url_args ""
catch {
    # This shouldn't fail, but why take chances?
    set url_args "$env(QUERY_STRING)"
}

if {[catch {libdb init -local -user $USER} myDb]} {
    # failed to initialize!!!
    lappend err_list "Unable to connect to database: ${myDb}"
    HTML_Start ${page_title} aaa_servers.tcl?$url_args
    show_result 1
    HTML_End
    set myDb {}
    exit 1
}

#initialize the cluster handle if we are running in CV
if {$is_cluster} {
   if {[catch {cluster init -cid $cid -user $USER} myCl]} {
      #failed to initialize!!!
      lappend err_list "Unable to connect to cluster: ${myCl}\n"

      # we dont want to show apply and save buttons when cluster failed.
      set APPLY_REQ false
      set SAVE_REQ false

      HTML_Start ${page_title} aaa_servers.tcl?$url_args
      show_result 1
      HTML_End
      libdb close $myDb
      set myCl {}
      exit 1
   }
}

set result 0

set method [get_Method] 

# Make sure we have some params that we need, and that they have reasonable
# values.  If not, abort before applying or displaying anything.
# Also, process some parameters to extract some globally useful information.
if {[catch {
    set DOC "[libdb get $myDb docdir]/security.htm"

    if {$module == "RADIUS"} {
	set module_human "RADIUS"
	set srvvar radius_srv
	set module_tcpudp "UDP"
    } elseif {$module == "TACPLUS"} {
	set module_human "TACACS+"
	set srvvar tacplus_srv
	set module_tcpudp "TCP"
    } else {
	error "Module type ${module} does not take \"server\" configuration."
    }
    if {$function == "auth"} {
	set function_long "authentication"
	set function_long_other "session management"
    } elseif {$function == "sess"} {
	set function_long "session management"
	set function_long_other "authentication"
    } else {
	error "Function type $function does not take \"server\" configuration."
    }
    set page_title "AAA ${module_human} [\
	    string toupper ${function} 0 0]. Servers Configuration"
    set subpfx "aaa:${function}_profile:${profile}"
    set pf [libdb get $myDb $subpfx]
    set md [libdb get $myDb "${subpfx}:${function}_type"]
    if {$pf == ""} {
	error "A nonexistent $function. profile seems to have been requested."
    }
    if {$md != $module} {
	error "The $function. profile requested doesn't seem to have the type specified."
    }

    # a subprefix of that one, is what's used throughout the rest of this
    set pfx "${subpfx}:$srvvar"

    # These three configurable items are present in some cases of the page,
    # and not in others.
    set has_maxtries 0 ; # does this have a 'maxtries' configurable?
    set has_sendrec 0 ; # does this have a 'sendrec' configurable?
    set has_waitconnect 0 ; # does this have a 'waitconnect' configurable?
    if {$module == "RADIUS"} {
	set has_maxtries 1
    }
    if {$function == "sess"} {
	set has_sendrec 1
	set has_waitconnect 1
    }
} res]} {
    # Print an error page.
    lappend err_list $res
    HTML_Start $page_title aaa_servers.tcl?$url_args
    HTML {
	This page could not be generated.  It should only be reached via
	a link from the page <a href="aaa.tcl">aaa.tcl</a>.  Perhaps it
	was invoked some other way, or perhaps the link is no longer
	relevant.  The following error message may contain more details.
	What you probably should do is reload the page
	<a href="aaa.tcl">aaa.tcl</a> and look for the link from
	that one to this one.
	<p>
    }
    show_result 1
    HTML_End
    exit 1
}

# What per-server config items are possible:
set cfg_fields {host port timeout}
# Default values of various fields in a "new" entry.  Alternates name-value.
set cfg_fields_defs {host "" port "" timeout 3}
# Some fields are present in some pages and not others.
if {$has_maxtries} {
    lappend cfg_fields maxtries
    lappend cfg_fields_defs maxtries 3
}
if {$has_sendrec} {
    lappend cfg_fields sendrec
    lappend cfg_fields_defs sendrec start
}
if {$has_waitconnect} {
    lappend cfg_fields waitconnect
    lappend cfg_fields_defs waitconnect no
}
# "secret" is also possible, but isn't handled quite the same way
# (it's not displayed); "priority" as well, but it's part of the
# db variable name, not a variable on its own.
array set cfg_fields_defs_a $cfg_fields_defs

if {$method == "POST"} {
    if {$apply == 1} {
	# (Possible) changes are being committed, via an "apply" or "save"
	# button.
	
	# 1. Perform validation.
	set result [Validate $Q_Data err_list]

	# 2. Take the form input and massage it into a set of "change
	# operations".

	# value_changes: Changing existing servers: (prio var val)*
	# cprio_changes: Changing priorities: (oldprio newprio)*
	# new_changes: Variables for new server to add: (shortname value)*
	# new_priority: Priority of new server to add
	# delete_priorities: Priority values for servers to delete
	set value_changes {}
	set cprio_changes {}
	set new_changes {}
	set new_priority {}
	set delete_priorities {}
	if {$result == 0} {
	    set pfx_split [split $pfx :]

	    foreach var $Q_Names {
		set vcomps [split $var :]
		if {[lindex $vcomps 0] != ""} {
		    # 2a. Variables being set by name: process these first
		    # (everything else will be a revision).
		    # Form of vcomps: {aaa *_profile <profile>
		    # <srvvar> <prio> <var>}
		    if {[lrange $vcomps 0 3] == "${pfx_split}"} {
			# $vcomps is a list "aaa ptype profile priority..."
			set vlast [lindex $vcomps 5]
			set vval [getVal $var]
			if {$vlast != "secret" || $vval != "" } {
			    lappend value_changes [lindex $vcomps 4] \
				    $vlast $vval
			}
			# (secret is a special case: its current value
			# does not appear in the form, so if no value
			# is present, it does *not* mean we're erasing
			# it, it means it just isn't changed)

			if {$vlast == "secret"} {
			    check_for_backslash $vval
			}
		    }
		} else {
		    # (something of the form :op:details)
		    set vcomps1 [lindex $vcomps 1]
		    if {$vcomps1 == "change_priority"} {
			# 2b. Changing the priority of an existing
			# setting.  Invoked via
			# ":change_priority:<pfx>:<oldprio> <newprio>";
			# the result is deletion of db entries under
			# the one priority, and re-creation under the
			# new priority.

			if {[lrange $vcomps 2 5] != "${pfx_split}"} {
			    # it isn't anything we know about; ignore it
			    continue
			}

			# what is the old priority?
			set oldprio [lindex $vcomps 6]
			# what is the new priority?
			set newprio [getVal $var]
			if {
			    $newprio == "" || $oldprio == "" ||
			    $newprio == $oldprio
			} {
			    # It appears that nothing was changed, or there
			    # was nothing to change.
			    continue
			}

			lappend cprio_changes $oldprio $newprio
		    } elseif {$vcomps1 == "new"} {
			# 2c. Adding a new entry.  Invoked via
			# ":new:<pfx>:*" where "*" is "priority"
			# plus the various fields, going into
			# "<pfx>:<priority>:*".

			if {[lrange $vcomps 2 5] != "${pfx_split}"} {
			    # it isn't anything we know about; ignore it
			    continue
			}
			set nvnam [lindex $vcomps 6]

			if {$nvnam == "priority"} {
			    # The priority of the new entry
			    set new_priority [getVal $var]
			} else {
			    if {$nvnam == "secret"} {
				check_for_backslash [getVal $var]
			    }
			    # Some parameter of the new entry
			    set value [getVal $var]
			    lappend new_changes $nvnam [getVal $var]
			}
		    } elseif {$vcomps1 == "delete"} {
			# 2d. Deleting an existing server config.
			# Variable name syntax ":delete:<prefix>:<priority>";
			# deletes on any non-null value.
			if {[lrange $vcomps 2 5] != "${pfx_split}"} {
			    # it isn't anything we know about; ignore it
			    continue
			}
			set dpri [lindex $vcomps 6]

			if {[getVal $var] != ""} {
			    # Yes, we are going to delete something!
			    lappend delete_priorities $dpri
			}
		    } else {
			# Invalid input.  This is not the user's fault;
			# it's probably misbehavior by either the browser
			# or, more likely, by this script.  In either case,
			# just ignore the bogus stuff.
		    }
		}
	    }
	}

	# 3. Take the set of "change operations" above, and figure
	# out what database operations it will take to
	# perform the desired actions.  This also checks for various
	# inconsistencies in the above, e.g. duplicate priority values.
	if {$result == 0} {
	    # dbchanges: variable-value pairs to go in db; variable names
	    # in this are Tcl lists of: {prio [field]}.  "prio" is old
	    # value which may be remapped, or removed, and may be "n" for
	    # new.
	    array set dbchanges {}
	    # pchanges1: priority change info: maps {prio action} to boolean
	    array set pchanges1 {}
	    # pmap: map old priorities (plus "n" for new) to new priorities
	    # (plus "d" for delete).
	    array set pmap {}

	    # 3a. The "secret" field gets special treatment.  It isn't
	    # set as a "default value" in the form, and for some of our
	    # later processing we want to know the value of "secret".
	    # So we take it from the database, right now.
	    foreach {p} [libdb iterate children $myDb ${pfx}] {
		set secval [libdb get $myDb "${pfx}:${p}:secret"]
		if {$secval != ""} {
		    arraymap_set dbchanges [list $p secret] $secval
		    arraymap_set pchanges1 [list $p chg] 1
		}
	    }

	    # 3b. Simple changes
	    foreach {vcpri vcvar vcval} $value_changes {
		arraymap_set dbchanges [list $vcpri $vcvar] $vcval
		arraymap_set pchanges1 [list $vcpri chg] 1
	    }

	    # 3c. Changing priorities of existing servers.  What we have
	    # at the beginning is a list of old and new priorities.
	    foreach {cpopri cpnpri} ${cprio_changes} {
		# Modify 'pmap' to map old priority to new.
		arraymap_set pmap $cpopri $cpnpri
		# Special db entry should be mapped too; it isn't already in
		# dbchanges.
		arraymap_set dbchanges [list $cpopri ""] "t"

		arraymap_set pchanges1 [list $cpopri pr-] 1
		arraymap_set pchanges1 [list $cpnpri pr+] 1
	    }
	    
	    # 3d. Deleting servers.  Removes every field of ones being
	    # deleted.
	    foreach dpri ${delete_priorities} {
		arraymap_set pmap $dpri d
		# also delete the entry "at the root of" the grouping
		# (We give it "t" here, but it will get "" in the end.
		# But that comes in at the "mapping" phase.)
		arraymap_set dbchanges [list $dpri ""] "t"
		arraymap_set pchanges1 [list $dpri del] 1
	    }

	    # 3e. Newly added servers.  If any entries are there,
	    # then we can add in the db.  We will also want to check for
	    # duplicate priorities, but that comes later.  Checking to
	    # make sure no entries are missing will be left to the CDK.
	    # (Except for the things the CDK would miss.)

	    # Look for things that "mean it was entered": a priority,
	    # or a value where there was no default, or a value different
	    # from the default.
	    set new_prio_pres 0
	    if {$new_priority != ""} {
		set new_prio_pres 1
	    }
	    set new_chgs_pres 0
	    foreach {n v} ${new_changes} {
		set defval [lindex [array get cfg_fields_defs_a $n] 1]
		if {$defval != $v && $v != ""} {
		    set new_chgs_pres 1
		}
	    }
	    if {$new_prio_pres || $new_chgs_pres} {
		# Make sure priority is present and one other thing;
		# the CDK does any other error checking.
		set emsg ""
		if {(!$new_prio_pres) || (!$new_chgs_pres)} {
		    set emsg "To configure a new server, you must specify"
		    set emsg "$emsg its priority, host address, shared"
		    if {$has_maxtries} {
			set emsg "$emsg secret, timeout, and maximum tries."
		    } else {
			set emsg "$emsg secret, and timeout."
		    }
		    lappend err_list $emsg
		    set result 1
		}
		
		# Prepare the database changes involved
		# "n" is a "fake" priority value mapped to the real one
		arraymap_set pmap n ${new_priority}
		foreach {sn v} ${new_changes} {
		    arraymap_set dbchanges [list "n" $sn] $v
		}
		# There's also a database change "at the root of" the entry
		arraymap_set dbchanges [list "n" ""] t
		# Record for priority checking later
		arraymap_set pchanges1 [list $new_priority new] 1
	    }

	    # 3f. Make sure we don't duplicate any priority values.
	    # Above, we recorded what's happening to various priorities.
	    # Now we process that information.
	    set prios {}
	    foreach {pa pabool} [arraymap_listall pchanges1] {
		if {$pabool == "1"} {
		    # $pa is {$p $a} where $p is a priority ($a we ignore
		    # for now)
		    lappend prios [lindex $pa 0]
		}
	    }
	    # make the priorities in the list unique...
	    set prios [luniq [lsort $prios]]
	    foreach p $prios {
		# $p is a priority.  Find out "what happens to" this priority
		# in the course of the configuration changes.
		set pexists 0
		if {[arraymap_get pchanges1 [list $p chg]] == "1"} {
		    # It was edited, so it existed before
		    set pexists 1
		}
		if {[arraymap_get pchanges1 [list $p del]] == "1"} {
		    # It was deleted.
		    if {$pexists} {
			# It existed, now it doesn't.
			set pexists 0
		    } else {
			# It didn't exist, couldn't be deleted.  But that's
			# not the user's fault (they shouldn't have seen the
			# delete button) so just skip it.
		    }
		}
		if {[arraymap_get pchanges1 [list $p pr-]] == "1"} {
		    # It was renamed to something else.
		    if {$pexists} {
			# It existed, now it doesn't.
			set pexists 0
		    } else {
			# It didn't exist, is meaningless to delete.
			# This case might come up when the user changes
			# the priority and chooses delete.  It's perfectly
			# OK.  We don't need to handle it specially.
		    }
		}
		foreach addby {pr+ new} {
		    if {[arraymap_get pchanges1 [list $p $addby]] == "1"} {
			# A server config at this priority was added in one
			# of two ways:
			#	pr+ -- renamed from something else
			#	new -- newly added server has this priority
			if {$pexists} {
			    # It already existsed; this is illegal.
			    set emsg "Your configuration was rejected because"
			    set emsg "$emsg it would result in two servers"
			    set emsg "$emsg configured with the same priority"
			    set emsg "$emsg value ($p)."
			    lappend err_list $emsg
			    set result 1
			} else {
			    # It didn't exist; now it does
			    set pexists 1
			}
		    }
		}
	    }

	    # 3g. Produce an "inverse priority map"
	    array set pmapinv {}
	    foreach {n v} [arraymap_listall pmap] {
		array set pmapinv [list $v $n]
	    }
	}

	# 4. Now, apply the changes we've built up above (provided there
	# were no problems).

	if {$result == 0} {
	    # build up a database command
	    set set_list {}
	    foreach {pri-fld val} [arraymap_listall dbchanges] {
		# $val is the variable's value.  ${pri-fld} is a list
		# containing the (old) priority code, and the field name.
		# $opri -- old priority (may be current too)
		# $npri -- new priority
		# $ppri -- even older priority
		set opri [lindex ${pri-fld} 0]
		set npri [arraymap_get pmap $opri]
		set ppri [arraymap_get pmapinv $opri]
		set fld [lindex ${pri-fld} 1]
		# map to the new priority
		if {$npri == ""} {
		    # Was not mapped.  Use existing value.
		    set npri $opri
		}
		if {$fld == ""} {
		    set colonfld ""
		} else {
		    set colonfld ":$fld"
		}
		if {$npri != $opri && $opri != "n" && $ppri == ""} {
		    # A server is deleted or moved, and nothing put in
		    # its place: delete the old db entries.
		    lappend set_list "${pfx}:${opri}$colonfld" ""
		}
		if {$npri != "d" && $npri != "n"} {
		    # A server is changed or reprioritied or added:
		    # add/change db entries.
		    lappend set_list "${pfx}:${npri}$colonfld" $val
		}
	    }
	    if {$save == "1"} {
		# and the user requested a save
		lappend set_list :save ""
	    }
	    if {[llength $set_list] > 0} {
		# apply it
                if {$is_cluster} {
                    set result [catch [concat {cluster db set $myCl -user $USER} $set_list] emsg]
                } else {
		    set result [catch \
			[concat {libdb set $myDb -list} $set_list] emsg]
                }
		# $emsg is now an error or a notification (which it is depends
		# on the result status)
		if {$emsg != ""} {
		    lappend err_list $emsg
		}
	    }
	}
    }
}

set headerbits "<script type=\"text/javascript\" src=\"http://o.aolcdn.com/dojo/1.0.0/dojo/dojo.xd.js\" 
        djConfig=\"parseOnLoad: true\"></script>
    <script type=\"text/javascript\">
       dojo.require(\"dojo.parser\");
       dojo.require(\"dijit.form.ComboBox\");
       dojo.require(\"dijit.layout.ContentPane\");
       dojo.require(\"dijit.layout.TabContainer\");
       dojo.require(\"dijit.layout.AccordionContainer\");
       dojo.require(\"dijit.form.Button\");

       function setVal1(value) {
           console.debug(\"Selected \"+value);
       }

        document.body.className=\"tundra\";
        var linkobj = document.getElementsByTagName(\"link\")\[0\];
        linkobj.href=\"/opt/HullPlating/web/voyager/voyagerCommon.css\";
</script>"

# the "config_top" replacement included here automatically
HTML_Start $page_title aaa_servers.tcl?$url_args

puts $headerbits

HTML_Link aaa.tcl "Return to HullPlated AAA config"
HTML_BR
HTML_BR

show_result $result

set ht "<B>${page_title}</B><BR><BR>"
set ht "$ht Configure ${module_human} servers for use in a single"
set ht "$ht ${function_long} profile.  Other profiles using ${module_human}"
set ht "$ht may be configured"
set ht "$ht separately, using pages similar to this one."
if {$function == "sess"} {
    set ht "$ht This page is where you configure ${function_long} servers"
    set ht "$ht for accounting (accounting falls under session management)."
}
set ht "$ht<P>"
set ht "$ht Each column in the table is a separate server; you may edit the"
set ht "$ht fields of existing servers, delete existing servers, or"
set ht "$ht configure additional servers.  If multiple servers are configured"
set ht "$ht for this profile, the client will attempt the ones with the"
set ht "$ht <i>smallest</i> priority number <i>first</i>, and only attempt"
set ht "$ht the others when and if the first server fails to respond.  If"
set ht "$ht you want the ${module_human} client to contact multiple servers"
set ht "$ht even when successful, this can be done by configuring multiple"
set ht "$ht ${function}. profiles."
set ht "$ht<P>"
set ht "$ht To add a new server to the configuration, enter its details"
set ht "$ht in the bottom row of the table.  To delete a server from"
set ht "$ht the configuration, check the box in that server&#39;s row"
set ht "$ht in the &quot;delete&quot; column."

HELP $ht

set prof_pfx $pfx

if {0} {
    # XXX get rid of this eventually
    # Some debug code
    if {$method == "POST" && [info exists set_list]} {
	HTML_Table "Database Operations" "name" "value"
	set set_list_2 {}
	foreach {n v} $set_list {
	    lappend set_list_2 [list $n $v]
	}

	foreach nv [lsort -index 0 $set_list_2] {
	    HTML_Row
	    HTML_Col
	    TEXT [lindex $nv 0]
	    HTML_Col
	    TEXT [lindex $nv 1]
	    HTML_EndRow
	}
	HTML_EndTable
    }
}

# RADIUS and TACACS+ have similar server configuration
# They have a table of servers, with option to delete any, to change
# parts of any, or to add one.

set headers {Priority "Host Address" "Port #" Secret Timeout}
# These are not always needed
if {$has_maxtries} {
    lappend headers "Max Tries"
}
if {$has_sendrec} {
    lappend headers "Send Records"
}
if {$has_waitconnect} {
    lappend headers "Wait for Connect"
}
lappend headers "Delete"
set table_title "${module_human} Servers for [\
	string toupper ${function} 0 0]. Profile \"$profile\""
eval [concat {HTML_Table ${table_title}} $headers]
# entries for existing servers
foreach priority [lsort -integer [libdb iterate children $myDb ${prof_pfx}]] {
    set srv_pfx "${prof_pfx}:${priority}"
    if {[libdb get $myDb ${srv_pfx}] == ""} {
	# Not really configured: so skip it (this should not happen)
	continue
    }
    
    # Get this server's settings
    set srv_host		[libdb get $myDb "${srv_pfx}:host"]
    set srv_port		[libdb get $myDb "${srv_pfx}:port"]
    set srv_secret		[libdb get $myDb "${srv_pfx}:secret"]
    set srv_timeout		[libdb get $myDb "${srv_pfx}:timeout"]
    # These are not always needed
    if {$has_maxtries} {
	set srv_maxtries		[libdb get $myDb "${srv_pfx}:maxtries"]
    }
    if {$has_sendrec} {
	set srv_sendrec	[libdb get $myDb "${srv_pfx}:sendrec"]
    }
    if {$has_waitconnect} {
	set srv_waitconnect	[libdb get $myDb "${srv_pfx}:waitconnect"]
    }
    
    # Display this server's settings (and provide means to change them)
    HTML_Row
    HTML_Col
    get_Int "" ":change_priority:${srv_pfx}" $priority \
	    "View/Set the priority of this server" 0
    HTML_Col
    get_IPv4_Unicast "" "${srv_pfx}:host" $srv_host \
	    "View/Set the address of the server host" 0
    HTML_Col
    get_Int_With_Range "" "${srv_pfx}:port" $srv_port 1 65535 \
	    "View/Set the ${module_tcpudp} port of the server (optional)" 1
    HTML_Col
    # Some browsers will limit the input to the width.  Thus we increase the
    # width to 16 ('secret' length recommended by the RFC) from its default
    # of 14.
    get_Passwd "" "${srv_pfx}:secret" -len 1 \
	    -maxlen $secret_maxlen -width $secret_width
    HTML_Col
    get_Positive_Int "" "${srv_pfx}:timeout" $srv_timeout \
	    "View/Set the timeout (in seconds)" 0
    # These are not always needed
    if {$has_maxtries} {
	HTML_Col
	get_Positive_Int "" "${srv_pfx}:maxtries" $srv_maxtries \
		"View/Set the maximum number of tries" 0
    }
    if {$has_sendrec} {
	HTML_Col
	HTML_Selection "" "${srv_pfx}:sendrec" 1 \
		start start stop stop $srv_sendrec
    }
    if {$has_waitconnect} {
	HTML_Col
	HTML_Selection "" "${srv_pfx}:waitconnect" 1 \
		yes yes no no $srv_waitconnect
    }
    HTML_Col
    HTML_CheckBox "" ":delete:${srv_pfx}" t
    HTML_EndRow
}
# The bottom row lets the user add a new entry.
HTML_Row
HTML_Col
get_Int "" ":new:${prof_pfx}:priority" "" "Priority of new server" 1
HTML_Col
get_IPv4_Unicast "" ":new:${prof_pfx}:host" \
	[lindex [array get cfg_fields_defs_a host]  1] \
	"Host address of new server" 1
HTML_Col
get_Int_With_Range "" ":new:${prof_pfx}:port" \
	[lindex [array get cfg_fields_defs_a port] 1] 1 65535 \
	"${module_tcpudp} port of new server" 1
HTML_Col
get_Passwd "" ":new:${prof_pfx}:secret" -len 1 \
	-maxlen $secret_maxlen -width $secret_width
HTML_Col
get_Positive_Int "" ":new:${prof_pfx}:timeout" \
	[lindex [array get cfg_fields_defs_a timeout] 1] \
	"Timeout (in seconds) for new server" 1
# These are not always needed
if {$has_maxtries} {
    HTML_Col
    get_Positive_Int "" ":new:${prof_pfx}:maxtries" \
	    [lindex [array get cfg_fields_defs_a maxtries] 1] \
	    "Maximum number of tries for new server" 1
}
if {$has_sendrec} {
    HTML_Col
    HTML_Selection "" ":new:${prof_pfx}:sendrec" 1 start start stop stop \
	    [lindex [array get cfg_fields_defs_a sendrec] 1]
}
if {$has_waitconnect} {
    HTML_Col
    HTML_Selection "" ":new:${prof_pfx}:waitconnect" 1 yes yes no no \
	    [lindex [array get cfg_fields_defs_a waitconnect] 1]
}
HTML_Col
# in the "new entry" row, there is no "delete" button; instead, indicate
# the fact.
HTML "<font color=\"green\">(new)</font>"
HTML_EndRow

HTML_EndTable

set ht "<B>Priority:</B>"
set ht "$ht A number indicating the priority of this server.  When multiple"
set ht "$ht servers are configured for a profile, the priority determines"
set ht "$ht which should be tried first. A smaller number indicates a higher"
set ht "$ht priority."
set ht "$ht<BR><BR><UL type=disc>"
set ht "$ht<LI><B>Range:</B> Any integer. Must be unique among the"
set ht "$ht servers configured for this ${function}. profile.</LI>"
set ht "$ht<LI><B>Default:</B> None (required field)</LI></UL>"

set ht "$ht<BR><B>Host Address:</B>"
set ht "$ht The IP address of the ${module_human} server."
set ht "$ht<BR><BR><UL type=disc>"
set ht "$ht<LI><B>Range:</B> IP (IPv4) unicast address in numeric,"
set ht "$ht dot-delimited format.</LI>"
set ht "$ht<LI><B>Default:</B> None (required field)</LI></UL>"

if {$module == "RADIUS"} {
    # RADIUS has different default ports for authentication and accounting.
    # Also, traditional port usage from many vendors differs from the standard.
    if {$function == "auth"} {
	set default_port 1812
	set trad_port 1645
    } else {
	set default_port 1813
	set trad_port 1646
    }
} else {
    set default_port 49
    set trad_port ""
}
set ht "$ht<BR><B>Port #:</B>"
set ht "$ht The ${module_tcpudp} port to contact on the server host."
set ht "$ht The correct value is determined by the configuration of your"
set ht "$ht ${module_human} server, which will not be detailed here."
if {$trad_port == ""} {
    set ht "$ht A common value (the default) is ${default_port}."
} else {
    set ht "$ht Common values are ${default_port} (the default, specified"
    set ht "$ht by the standard) and ${trad_port} (nonstandard, but"
    set ht "$ht used traditionally)."
}
set ht "$ht<BR><BR><UL type=disc>"
set ht "$ht<LI><B>Range:</B> Integers 1-65535.</LI>"
set ht "$ht<LI><B>Default:</B> ${default_port} (applies if field is empty)."
set ht "$ht</LI></UL>"

set ht "$ht<BR><B>Secret:</B>"
set ht "$ht The &quot;shared secret&quot; used to authenticate this"
set ht "$ht ${module_human} server, and the local client, to each other."
set ht "$ht The same value should be configured on your ${module_human}"
set ht "$ht server (how to do that, will not be detailed here). Good"
set ht "$ht secrets are ones that are hard to guess, like passwords."
set ht "$ht We strongly recommend, however, that you do <em>not</em>"
set ht "$ht reuse your actual password, or any other, as a shared secret,"
set ht "$ht because secrets are stored in retrievable form in two places"
set ht "$ht (the client and the server)."
if {$module == "RADIUS"} {
    set ht "$ht RFC 2865 recommends that"
    set ht "$ht the secret be at least 16 characters in length.  Some"
    set ht "$ht RADIUS servers limit the secret to at <em>most</em>"
    set ht "$ht 16 characters in length (in some cases, 15)."
}
set ht "$ht<BR>For security reasons, the configured secrets are <em>not</em>"
set ht "$ht displayed in this table.<BR><BR><UL type=disc>"
set ht "$ht<LI><B>Range:</B> Text strings, without backslash or newline.</LI>"
set ht "$ht<LI><B>Default:</B> None (field is required).</LI></UL>"

set ht "$ht<BR><B>Timeout:</B>"
set ht "$ht The number of seconds to wait, after contacting this server,"
set ht "$ht for a response.  If no usable response is received in this time,"
if {$has_maxtries} {
    set ht "$ht the client may retry, or move on to another server, or give up"
    set ht "$ht (as configured)."
} else {
    set ht "$ht the client may move on to another server or give up"
    set ht "$ht (as configured)."
}
set ht "$ht<BR><BR><UL type=disc>"
set ht "$ht<LI><B>Range:</B> Positive integers.</LI>"
set ht "$ht<LI><B>Default:</B> 3.</LI></UL>"

if {$has_maxtries} {
    set ht "$ht<BR><B>Max Tries:</B>"
    set ht "$ht The number of attempts to make to contact this server."
    set ht "$ht If no attempt receives a usable response within the"
    set ht "$ht timeout period the client will give up.  Note that this"
    set ht "$ht is not exactly a <em>retry</em> limit:  It includes"
    set ht "$ht the first attempt.  So a value of &quot;3&quot; in this"
    set ht "$ht field means two retries.<BR><BR><UL type=disc>"
    set ht "$ht<LI><B>Range:</B> Positive integers.</LI>"
    set ht "$ht<LI><B>Default:</B> 3.</LI></UL>"
}

if {$has_sendrec} {
    set ht "$ht<BR><B>Send Records:</B>"
    set ht "$ht Indicates whether both &quot;start&quot; and &quot;stop&quot;"
    set ht "$ht accounting records should be sent to this server, or only"
    set ht "$ht &quot;stop&quot; records.<BR><UL type=disc><LI><B>Options:</B>"
    set ht "$ht<UL><LI><B>start:</B>"
    set ht "$ht Client should send both kinds of records. (default)</LI>"
    set ht "$ht<LI><B>stop:</B>"
    set ht "$ht Client should send only &quot;stop&quot; records.</LI>"
    set ht "$ht</UL></LI></UL>"
}
if {$has_waitconnect} {
    set ht "$ht<BR><B>Wait for Connect:</B>"
    set ht "$ht Indicates whether the accounting client will wait around"
    set ht "$ht for responses from accounting servers.<BR><BR><UL type=disc>"
    set ht "$ht<LI><B>Options:</B><UL>"
    set ht "$ht<LI><B>Yes:</B> Client <em>should</em> wait.</LI>"
    set ht "$ht<LI><B>No:</B> Client should <em>not</em> wait.</LI></UL>"
    set ht "$ht<LI><B>Default:</B> No.</LI></UL>"
}

set ht "$ht<BR><B>Delete:</B>"
set ht "$ht The checkboxes in this column allow you to delete server"
set ht "$ht configurations from the table.  The checkbox does not appear"
set ht "$ht in the table&#39;s bottom row, because that row is for entering"
set ht "$ht <em>new</em> server configurations."

HELP $ht

libdb close $myDb

# the "config_bottom" replacement included here automatically
HTML_End

exit

