#! /usr/bin/ipstcl
######################################################################
# Copyright 1995-2002 Nokia IP, Inc.  All rights reserved.
#
# YOU MAY ONLY USE THE MATERIALS FOUND IN THIS FILE PURSUANT TO THE
# TERMS OF THE EVALUATION LICENSE AGREEMENT FOUND IN "/LICENSE".  ALL
# OTHER USES ARE EXPRESSLY PROHIBITED.
######################################################################
#
# $Id: check_cluster.tcl,v 1.19 2006/06/24 02:21:07 dilatush Exp $
#


#this function returns the CID if the clustering is
#On. Else returns -1

proc get_cluster_id {} {
	global env
	if {[info exists env(CLUSTER_ID)]} {
		return $env(CLUSTER_ID)
	} else {
		return -1
	}
}

#this function if cluster is on. Returns 1 if it is
#on; else 0
proc check_cluster {} {
	global env
	global USER
	if {[info exists env(CLUSTER_ID)]} {
		return 1
	} else {
		return 0
	}
}

proc print_no_cluster_error {} {
	puts "<HTML>"
	puts "<BODY>"
	puts "Not in clustering"
	puts "</BODY>"
	puts "</HTML>"
	
}

#
# Function to print warning features for both cloned and non-cloned feature.
# Print warning message 1 if through cluster Voyager and not a cloned feature
#                       User provides cluster id here
#        example: print_feature_warning "dns" $myDb $cid
# Print warning message 2 if non-cluster Voyager and is a cloned feature
#                       User need not provied cluster id here
#        example: print_feature_warning "dns" $myDb ""
# Note 1: myDb should be initialized
# Note 2: This function should be called before show_result is called.
# Note 3: Assumes err_list declared before calling this function.
#

proc print_feature_warning { feature myDb cid } {
    global err_list
    if {$cid != "" } {
        #
        # Don't have to print any warning on Cvoyager.
        #
	    return
    }
    set cluster_list [libdb iterate $myDb -c on "cluster:cid"]
    if { [llength $cluster_list] == 0 } {
        #
        # Don't have to print any warning if not in cluster.
        #
	   return
    }
    foreach id $cluster_list { 
        lappend err_list " This feature is associated with cluster id $id. "
        lappend err_list  \
          " Any changes made would be local to this cluster member only. "
        lappend err_list \
          " The changes may be overwritten by cluster configuration. "
    }

    return 
}


#
# Get the Cluster Nodes in the cluster
# Doesn't use myCl. May be used in future
# 
proc get_cluster_nodes {cid myDb myCl} {
    set nodes [list]
    if  { [file  exists  "/var/tmp/cluster_debug" ] == 1 } {
	set nodes [ libdb iterate children $myDb cluster:cid:$cid:member ]
    }  else {
	foreach node [ libdb iterate children $myDb ipsctl:net:ip:cluster:id:$cid:member ] {
	    set node_addr [ libdb get $myDb ipsctl:net:ip:cluster:id:$cid:member:$node:address ]
	    lappend nodes $node_addr
	}
    }
    return $nodes
}

# this proc creates a hash table out of default response
# the response is keyed on node id
# uretlist is the result array
# mk_list is to make list out of every node's multiline response
# Also returns 1 if all the values are same
proc cluster_hash_get {myCl prefix options uretlist mk_list} {
    upvar $uretlist retlist
    set cmd "cluster db get $myCl $options $prefix"
    set tmp [eval $cmd]
    set nlist [split $tmp "\n"]
    set nname ""
    array set retlist {}
    foreach nl $nlist {
        if {[string first "Response from node" $nl] != -1} {
            # start of a new data
            set nname [string range $nl [string length "Response from node "] end]
            set retlist($nname) ""
        } elseif {$nname != "" && $nl != ""} {
            # data for that node
            set nl [string trim $nl]
            set retlist($nname) "$retlist($nname)${nl}${mk_list}"
        }
    }
    set all_same 1
    set last_val ""
    set cnt 0
    foreach nd [array names retlist] {
        if {$cnt != 0} {
            if {$last_val != $retlist($nd)} {
                set all_same 0
            }
        }
        set last_val $retlist($nd)
        set cnt [expr $cnt + 1]
    }
    return $all_same
}

# From a string like nd: prefix, separate the cnode identifier
# and the prefix part
proc get_cnode_prefix {var ucnode uprefix} {
    upvar $ucnode cnode
    upvar $uprefix prefix
    if {[string first cnode $var] == 0} {
        set ind1 [string first ":" $var]
        set tmp [string range $var [expr $ind1 + 1 ] end]
        set ind2 [string first ":" $tmp]
        set nd [string range $var 0 [expr $ind1 + $ind2]]

        set cnode [string range $var [expr $ind1 + 1 ] [expr $ind1 + $ind2]]

        set prefix [string range $tmp [expr $ind2 + 1 ] end]
        return "${nd}:"
    } else {
        set prefix $var
        set cnode ""
        return ""
    }
    
} 

# check if all the values in udata hash are empty 
proc is_all_empty_vals {udata}  {
    upvar $udata data
        
    foreach nd [array names data] {
        if {$data($nd) != ""} {
            return 0
        }
    }
    return 1
}


######################################################################
# Function: run_cli
#
# Returns: 1 on failure, 0 on success
#
# cmd:		   is the cli command to run. The cluster should
#		   already be initialized and the input command should
#		   have the format: cluster cli $cid ?options? $cmd
#
# The output is separated by nodes and entered in a user given hash
# 'uretlist'. If no nodes information if found in the response. The 
# entire output is put in uretlist(0).
######################################################################
proc run_cli {cmd uretlist separator} {

    upvar $uretlist retlist

    if {[catch {eval $cmd} output]} {      
        lappend retlist $output                               
        return 1
    }

    set outputlines [split $output "\n"]

    array set retlist {}
    set nodename ""

    if {[string first "Response from node" $output] != -1} {
        foreach line $outputlines {
            if {[string first "Response from node" $line] != -1} {
                set nodename [string range $line \
                          [string length "Response from node "] end]
            } elseif {$nodename != "" && $line != ""} {
                set line [string trim $line]
                if {[info exists retlist($nodename)]} {
                    append retlist($nodename) "${separator}${line}"
                } else {
                    set retlist($nodename) $line
                }
            }
	}
    } else {
        foreach line $outputlines {
            set line [string trim $line]
            if {[info exists retlist(0)]} {
                append retlist(0) "${separator}${line}"
            } else {
                set retlist(0) $line
            }
        }
    }

    return 0
}


# combine_output proc takes output from multiple nodes
# and combines it into one big string
# it peforms the following
# Append each line with node identifier
# combine the lists
# sort them

proc combine_output {uoutput}  {
    upvar $uoutput output
    set ret [list]
    foreach clnode [array names output] {
        regsub -all "\n" $output($clnode) ":From:${clnode}\n" tmp
        set ret [concat $ret [split $tmp "\n"]]
    }
    
    set out [join [lsort $ret] "\n"]
    return $out
}

proc get_errs {uerr_list data} {
    upvar $uerr_list err_list
    upvar $data output

    foreach node [array names output] {
        if {$output($node) != ""} {
            lappend err_list "Node $node: $output($node)"
        }
    }
}

proc format_Response {response uretlist separator} {
    upvar $response Response
    upvar $uretlist retlist

    if {[info exists Response] == 0 || $Response == ""} {
        return 1
    }

    set outputlines [split $Response "\n"]

    array set retlist {}
    set nodename ""

    if {[string first "Response from node" $Response] != -1} {
        foreach line $outputlines {
            if {[string first "Response from node" $line] != -1} {
                set nodename [string range $line \
                          [string length "Response from node "] end]
            } elseif {$nodename != "" && $line != ""} {
                set line [string trim $line]
                if {[info exists retlist($nodename)]} {
                    append retlist($nodename) "${separator}${line}"
                } else {
                    set retlist($nodename) $line
                }
            }
	}
    } else {
        set retlist(0) ""
        foreach line $outputlines {
            set line [string trim $line]
            if {[info exists retlist(0)]} {
                append retlist(0) "${separator}${line}"
            } else {
                set retlist(0) $line
            }
        }
    }

    return 0
}
#
# Clone List: 
# Arranged in alphabetical order
# Should be in Sync with cluster_management_shared_features
# list defined in rconfig_clone.c
#
set clone_list {
     	"arp" "ARP" \
	"autosupport" "System Failure Notification" \
        "bgp" "BGP" \
        "bootp" "BOOTP Relay" \
	"cron" "Job Scheduler" \
	"dns" "DNS"\
	"hosts" "Host Address Assignment" \
	"http"  "Voyager Web Access" \
        "inboundfilters" "Inbound Route Filters" \
	"mailrelay" "Mail Relay" \
	"ntp" "Network Time Protocol" \
	"ospf" "OSPF" \
	"password-controls" "Password and Account Management Controls"
        "pim" "PIM" \
        "igmp" "IGMP" \
	"redistribution" "Route Redistribution" \
	"routingoptions" "Routing Options" \
  	"routemap" "Routemaps" \
	"services" "Network Access and Services" \
	"snmp" "SNMP" "ssh" "SSH" \
	"static" "Static Routes" \
	"time" "Local Time Setup"
}
